; This is a boot program for 80c32 microcontroller
; The main task of this program is to initialize all SFRs
; and then wait for a command passed over serial link
; from a PC.
; Once a command is received it has to be processed.
; 
; List of implemented commands
; 
; l - download a file from the host PC. File must be in bin format
;
; r - run downloaded code. As a convention downloaded code must have entry point at 400h
;
; t - toggle. This is test command. When received P1.7 is toggled.
;
;
; We have 8K of memory available. Generalized memory map is as following:
; 0h-3ffh - Boot area
; 400h-1fffh - User program area.
;
; PC after reset points to 0h, we need to redirect it.
; Also some more lower addresses are designated for special purporses,
; we need to count that:
; 0003h - interrupt vector address for IE0
; 000bh - TF0(timer0)
; 0013h - IE1
; 001bh - TF1
; 0023h - RI+TI (serial port)
; 002bh - TF2+EXF2

   
   	ljmp start	;jump to bootinit program 
   	ds 8		; will not use IE0
   	ljmp	t0_int_handler	;jump to T0 interrupt handler
   	ds 5		; Some space left
   	ds 8		; will not use IE1
   	ds 8		; will not use T1
   	ljmp sp_int_handler	; jump to Serial port interrupt handler
   	ds 5		; Some space left
   	ds 8 	; Will not use T2 interrupts
   
   ; Include the file containing various symbols and strings. All the symbol data will
   ; reside before 'start' routine
   

sp_int_handler:
   ; Because this is the bootinit routine, it's main purpose to download another program
   ; That's why we expect only RI interrupts. If something has to be sent back to the 
   ; host PC, Serial port interrupts must be disabled
   ;
   ; Communication protocol is such that we process only one symbol, but after cr is received.
   ; If multiple symbols were typed from the host PC prior pressing ENTER, the last symbol 
   ; will be processed
   ;
   ; @r0 will store the byte received from  PC   
   ;
   ; BDATA bits mapped as follows (event happened if bit is set):
   ; 0 - l received (download command)
   ; 1 - r received (run command)
   ; 2 - t received (toggle command)
    
   	jnb ri,sp_int_handler_exit	; If RI isn't set just exit
	mov a,sbuf			; Move received byte into A
	clr ri				; Acknowledge RI interrupt
	mov sbuf,a			; Echo received symbol
	cjne a,#cr,sp_handler_after_cr	; First of all check if pressed symbol is CR
	
	acall cmd_parse	; See what to process
	jmp sp_int_handler_exit
sp_handler_after_cr:
   	cjne a,#lf,sp_handler_after_lf
   ; If LF received,  don't copy the symbol
	jmp sp_int_handler_exit
sp_handler_after_lf:
   ; Everything else except from CR and LF just copied into r0
   	mov r0,a
	jmp sp_int_handler_exit
	
sp_int_handler_exit:
	reti		; Return from sp_int_handler
   	
	$INCLUDE(symbols_t2.h) 	; file contains symbol definitions

start:
   ; Initialize SFRs 
   	mov tmod,#21h	; Timer/Counter0-1 Mode Control Register. High nibble - T1, lower - T2
   		; T1 set to 8bit auto-reload, T0 - 16 bit mode
   	mov tcon,#0h	; Turn T1 off, T0 off
	mov t2con,#34h	; Use T2 as baud rate generator 
	mov rcap2h,#0ffh	; Load baud rate 0ffh for 9600bps
	mov rcap2l,#0f3h	; 0d1h for 9600bps (with 14.31818MHz crystal)
				; 0fch for 115200bps
				; 0f7h for 57600bps and 16MHz crystal
				; 0f3h for 38400bps
				; 0cch for 9600bps
   	mov scon,#50h	; Serial port mode1,enable serial reception
   	mov pcon,#0h	; SMOD reset
   	mov ip,#10h		; Set Serial port Interrupt priority to 1, all the rest - 0
   ; Initialize BDATA, which reflects status of the commands being received
   	mov 020h,#0	; Clear first 8 bits
   ; We need to make sure Serial port interrupt vector points to the correct ISR
   ; This may be changed by user program. That's why during booting up we need to restore it.
   ; Otherwise we won't be able to boot
   ; We need to move hex code of instruction 'ljmp sp_int_handler' to 023h
    	mov dptr,#023h
   	mov a,#02h
   	movx @dptr,a
   	inc dptr
   	mov a,#0h
   	movx @dptr,a
   	inc dptr
   	mov a,#033h
   	movx @dptr,a
	
   ;Before enabling interrupts print banner to the terminal using polling
   ;After SFRs are initialized we need to print banner to the terminal and the cprompt
   ; For this purporse disable interrupts from Serial Port
   	mov dptr,#greeting ; We'll present banner as LUT, dptr contains it's base
   	mov r3,#0	; Will be software counter
banner_print_start:
   	mov a,r3	; Load index of LUT, which is the banner we want to print, into A
   	movc a,@a+dptr
   	jz banner_printed	; All the symbols has been printed
   	mov sbuf,a	; If symbol is not NULL send it to SBUF
   	inc r3	; Point DPTR to the next byte
banner_print_tx_wait:
   	jnb ti,banner_print_tx_wait 	; Poll TI bit to see when transmission is complete
   	clr ti			; Clear flag
   	jmp banner_print_start	; Jump to print another symbol
   
banner_printed:
   ; After banner is printed we need to print cprompt
   	mov sbuf,#cprompt
   ; Again wait until transmission is complete
banner_printed_wait:
   	jnb ti,banner_printed_wait
   	clr ti
   ; Transmission is complete, just go ahead
   	mov ie,#90h	; Enable Serial Port Interrupts 
	setb ti		; Set Tx Int to prevent Tx interrupts
	cpl p1.7	; Toggle LED to switch it on	
   ; Now we just enter endless loop and wait for interrupts from Serial port
main_loop:
   ; During main loop we check BDATA to see if anything has been received from the host
   	jbc 0,l_main
main_after_l:
	jbc 1,r_main
main_after_r:
	jmp main_loop	; Endless loop

	
r_main:
	mov ie,#0	; Disable all interrupts
	lcall 0400h	; All downloaded programs must start from this address
			; and be completed by 'ret' instruction
	jmp main_after_r	; At the end of r_main routine

l_main:
   ; We need to download binary file to the memory starting at 0800h
   ; Timeout from T0 will be used to terminate downloading as binary file doesn't have
   ; any mark showing where EOF is
	mov ie,#0	; Disable interrupts from Serial port, polling will be used
   ; Print dprompt to the terminal
   	clr ti		; Clear TI Int before start as it was set previously
			; to echo symbols without being interrupted
   	mov sbuf,#cr
l_main_dprompt_cr:
	jnb ti,l_main_dprompt_cr	; Wait
	clr ti

	mov sbuf,#lf
l_main_dprompt_lf:
	jnb ti,l_main_dprompt_lf	; Wait
	clr ti

	mov sbuf,#dprompt
l_main_dprompt_d:
	jnb ti,l_main_dprompt_d		; Wait
	clr ti
	
	mov dptr,#0400h		; Init dptr with start address 
l_main_first_data:
	jnb ri,l_main_first_data	; Wait for the first data byte
	clr ri		; Acknowledge reception
	mov a,sbuf	; Copy to A
	movx @dptr,a	; Move first byte
	inc dptr	; DPTR points to the next byte
   ; For the rest bytes we need to init T0 to be used as timeout timer
   	mov th0,#th0_load
	mov tl0,#tl0_load	; Load counter registers
	setb tr0 	; Turn T0 on
l_main_rest_data:	
	jbc ri, l_main_copy_rest_data	; Check if a byte is received
	jbc tf0,l_main_timeout	; Check if T0 times out
	jmp l_main_rest_data

l_main_copy_rest_data:
	mov a,sbuf	; Copy received byte to accumulator
	movx @dptr,a
	inc dptr	; Increment DPTR
	clr tr0		; Stop T0
	mov th0,#th0_load
	mov tl0,#tl0_load	; Reload T0
	setb tr0	; Turn T0 back on
	jmp l_main_rest_data	; Look for another byte
	
l_main_timeout:
   ; All the data is copied
   	clr tr0		; Stop T0
   ; Now we need to print cprompt back instead of dprompt
   	mov sbuf,#cr
l_main_cprompt_cr:
	jnb ti,l_main_cprompt_cr	; Wait
	clr ti	

	mov sbuf,#lf
l_main_cprompt_lf:
	jnb ti,l_main_cprompt_lf	; Wait
	clr ti

	mov sbuf,#cprompt
l_main_cprompt_c:
	jnb ti,l_main_cprompt_c	; Wait
	clr ti

	mov ie,#090h		; Enable Serial Port interrupts
	setb ti			; Again set Tx Int bit to prevent Tx interrupts
	jmp main_after_l	; At the end of l_main routine

	
t0_int_handler:
	reti


cmd_parse:
   ; Check what's in r0
   	cjne r0,#06ch,cmd_parse_after_l
	acall handle_l
	jmp cmd_parse_exit
cmd_parse_after_l:
	cjne r0,#072h,cmd_parse_after_r
	acall handle_r
	jmp cmd_parse_exit
cmd_parse_after_r:
	cjne r0,#074h,cmd_parse_after_t
	acall handle_t
	jmp cmd_parse_exit
	
cmd_parse_after_t:
	jmp cmd_parse_exit	; No more commands implemented
cmd_parse_exit:	
	ret 	; Return from command_parse

handle_l:
   ; Set bit 0 in BDATA area to let main routine know that host requested 'l' command
   	setb 0
	ret

handle_r:
   ; Let main routine know about 'r' command request
   	setb 1
	ret
handle_t:
   ; This command toggles P1.7
	cpl p1.7
	ret	; Return from handle_t

	
short_delay:
   ; This subroutine implements short delay
   	mov r6,#0ffh
	djnz r6,short_delay
	ret	; return from short_delay

	
long_delay:
   ; This subroutine implements long delay
	mov r7,#010h 	; This actually specifies how many times the long delay is longer that
		; the short delay
	djnz r7,long_delay_int_loop
	jmp long_delay_exit
long_delay_int_loop:	
	mov r6,#0ffh
	djnz r6,long_delay_int_loop
	jmp long_delay
long_delay_exit:
	ret	; return from long_delay
	
   end	
